//
// (c) 2006 DS Data Systems UK Ltd, All rights reserved.
//
// DS Data Systems and KonaKart and their respective logos, are 
// trademarks of DS Data Systems UK Ltd. All rights reserved.
//
// The information in this document is free software; you can redistribute 
// it and/or modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//

package com.konakart.bl.modules.ordertotal.subtotal;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.torque.TorqueException;

import com.konakart.app.KKConfiguration;
import com.konakart.app.KKException;
import com.konakart.app.Order;
import com.konakart.app.OrderTotal;
import com.konakart.appif.KKEngIf;
import com.konakart.bl.modules.BaseModule;
import com.konakart.bl.modules.ordertotal.BaseOrderTotalModule;
import com.konakart.bl.modules.ordertotal.OrderTotalInterface;
import com.konakart.bl.modules.ordertotal.OrderTotalMgr;
import com.workingdogs.village.DataSetException;

/**
 * Module that creates an OrderTotal object for the subtotal amount of an order which could include
 * or exclude tax depending on how KonaKart is configured.
 * 
 */
public class Subtotal extends BaseOrderTotalModule implements OrderTotalInterface
{

    private static int sortOrder = -1;

    private static String code = OrderTotalMgr.ot_subtotal;

    private static String bundleName = BaseModule.basePackage + ".ordertotal.subtotal.Subtotal";

    private static HashMap<Locale, ResourceBundle> resourceBundleMap = new HashMap<Locale, ResourceBundle>();

    private static String mutex = "otSubtotalMutex";

    // Configuration Keys

    private final static String MODULE_ORDER_TOTAL_SUBTOTAL_SORT_ORDER = "MODULE_ORDER_TOTAL_SUBTOTAL_SORT_ORDER";

    private final static String MODULE_ORDER_TOTAL_SUBTOTAL_STATUS = "MODULE_ORDER_TOTAL_SUBTOTAL_STATUS";

    // Message Catalogue Keys

    private final static String MODULE_ORDER_TOTAL_SUBTOTAL_TITLE = "module.order.total.subtotal.title";

    // private final static String MODULE_ORDER_TOTAL_SUBTOTAL_DESCRIPTION =
    // "module.order.total.subtotal.description";

    /**
     * Constructor
     * 
     * @param eng
     * 
     * @throws DataSetException
     * @throws KKException
     * @throws TorqueException
     * 
     */
    public Subtotal(KKEngIf eng) throws TorqueException, KKException, DataSetException
    {
        super.init(eng);

        // Create the static maps from the configuration info
        if (sortOrder == -1)
        {
            synchronized (mutex)
            {
                if (sortOrder == -1)
                {
                    setStaticVariables();
                }
            }
        }
    }

    /**
     * Sets some static variables during setup
     * 
     * @throws KKException
     * 
     */
    public void setStaticVariables() throws KKException
    {
        KKConfiguration conf;

        conf = getEng().getConfiguration(MODULE_ORDER_TOTAL_SUBTOTAL_SORT_ORDER);
        if (conf == null)
        {
            sortOrder = 0;
        } else
        {
            sortOrder = new Integer(conf.getValue()).intValue();
        }
    }

    /**
     * Returns true or false
     * 
     * @throws KKException
     */
    public boolean isAvailable() throws KKException
    {
        return isAvailable(getEng(), MODULE_ORDER_TOTAL_SUBTOTAL_STATUS);
    }

    /**
     * Create and return an OrderTotal object for the subtotal amount.
     * 
     * @param order
     * @param dispPriceWithTax
     * @param locale
     * @return Returns an OrderTotal object for this module
     * @throws Exception
     */
    public OrderTotal getOrderTotal(Order order, boolean dispPriceWithTax, Locale locale)
            throws Exception
    {
        OrderTotal ot;

        // Get the resource bundle
        ResourceBundle rb = getResourceBundle(mutex, bundleName, resourceBundleMap, locale);
        if (rb == null)
        {
            throw new KKException("A resource file cannot be found for the country "
                    + locale.getCountry());
        }

        ot = new OrderTotal();
        ot.setClassName(code);
        ot.setSortOrder(sortOrder);
        BigDecimal subTotal;
        if (dispPriceWithTax)
        {
            subTotal = order.getSubTotalIncTax();
        } else
        {
            subTotal = order.getSubTotalExTax();
        }

        ot.setTitle(rb.getString(MODULE_ORDER_TOTAL_SUBTOTAL_TITLE) + ":");
        ot.setText(getCurrMgr().formatPrice(subTotal, order.getCurrencyCode()));
        ot.setValue(subTotal);
        return ot;
    }

    public int getSortOrder()
    {
        return sortOrder;
    }

    public String getCode()
    {
        return code;
    }
}
